/*
 *  nvj - NattoView/Java 
 *  (c) 2005-2007 Hidekazu Shiozawa
 *  http://vilab.org
 */

import java.io.*;
import java.net.*;
import java.util.*;
import java.awt.*;
import javax.swing.text.*;
import javax.swing.text.html.*;
import javax.swing.text.html.parser.*;

public class MixiChart extends NvChart {

    String email = null;
    String password = null;
    String BF_SESSION = null;
    String BF_STAMP = null;
    
    boolean allThumnails = false;
    
    public MixiChart(NvLayout layout, String email, String password) {
        super(layout);
        this.email = email;
        this.password = password;
        this.comparator = new Comparator<NvNode>() {
            public int compare(NvNode v1, NvNode v2) {
                return ((Integer)v1.getId()).intValue() - ((Integer)v2.getId()).intValue();
            }
        };
    }

    protected NvNode createNode(Object id, String label) {
        MixiNode node = new MixiNode(this, (Integer)id, label);
        return node;
    }

    protected NvLink createLink(NvNode source, NvNode dest, String label) {
        return new MixiLink((MixiNode)source, (MixiNode)dest, label);
    }

    public boolean login() {
        URL url = null;
        HttpURLConnection conn = null;
        try {
            String post = "email=" + email + "&password=" + password
                + "&next_url=/home.pl\r\n";

            url = new URL("http://mixi.jp/login.pl");
            conn = (HttpURLConnection)url.openConnection();
            conn.setRequestMethod("POST");
            conn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
            conn.setRequestProperty("Content-Length", "" + post.length());
            conn.setRequestProperty("Referer", "http://mixi.jp");
            conn.setDoOutput(true);
            PrintWriter out = new PrintWriter(
                    new OutputStreamWriter(conn.getOutputStream()));
            out.println(post);
            out.flush();
            String key, field;
            System.err.println(conn.getHeaderField(0));
            for (int i = 1; (key = conn.getHeaderFieldKey(i)) != null; i++) {
                field = conn.getHeaderField(i);
                if (key.equals("Set-Cookie")) {
                    String cookie = field.substring(0, field.indexOf(";"));
                    if (field.indexOf("BF_SESSION") >= 0)
                        BF_SESSION = cookie;
                    else if (field.indexOf("BF_STAMP") >= 0)
                        BF_STAMP = cookie;
                    System.err.print("*");
                }
                System.err.println(key + ": " + field);
            }
            if (BF_SESSION == null && BF_STAMP == null)
                return false;
        } catch (Exception e) {
            return false;
        }
        return true;
    }
    
    public void startScanningLinks(NvNode node) {
        LinkScanner scanner = new LinkScanner((MixiNode)node);
        new Thread(scanner).start();
    }

    class LinkScanner implements Runnable {
        MixiNode source;
        
        LinkScanner(MixiNode node) {
            this.source = node;
        }
        
        public void run() {
            source.lastScan = System.currentTimeMillis();
            if (BF_SESSION == null || BF_STAMP == null) {
                boolean ok = login();
                if (!ok) return;
            }
            for (int page = 1; scan(page) == 50; page++)
                ;
            layout();
        }

        private int scan(int page) {
            try {
                final URL url = new URL("http://mixi.jp/list_friend.pl?page=" + page
                        + "&id=" + source.getId().toString());
                HttpURLConnection conn = (HttpURLConnection)url.openConnection();
                conn.setRequestMethod("GET");
                conn.setRequestProperty("Referer", "http://mixi.jp/login.pl");
                conn.setRequestProperty("Cookie", BF_SESSION + "; " + BF_STAMP);
                conn.setRequestProperty("User-Agent", NvSettings.userAgent);
                System.err.println(BF_SESSION);
                System.err.println(BF_STAMP);

                String key, field;
                for (int i = 1; (key = conn.getHeaderFieldKey(i)) != null; i++) {
                    field = conn.getHeaderField(i);
                    if (key.equals("Set-Cookie")) {
                        if (field.indexOf("BF_SESSION") >= 0)
                            BF_SESSION = field;
                        else if (field.indexOf("BF_STAMP") >= 0)
                            BF_STAMP = field;
                        System.err.print("*");
                    }
                    System.err.println(key + ": " + field);
                }

                BufferedReader in = new BufferedReader(new InputStreamReader(
                        conn.getInputStream(), "JISAutoDetect"));
                System.err.println("" + url + " : analyzing...");
                
                MixiParserCallback parserCallback = new MixiParserCallback(source);
                new ParserDelegator().parse(in, parserCallback, true);

                in.close();
                conn.disconnect();
                System.err.println("" + url + " : done!");
                return parserCallback.mymixies;
            } catch (Exception e) {
                e.printStackTrace();
                return 0;
            }
        }
    }

    class MixiParserCallback extends HTMLEditorKit.ParserCallback {
        MixiNode source;
        int mymixies = 0;
        
        boolean facerow = false;
        Queue<Integer> idList = new LinkedList<Integer>();
        Queue<String> imageList = new LinkedList<String>(); 
        StringBuffer text = null;

        MixiParserCallback(MixiNode node) {
            this.source = node;
        }
        
        public void handleStartTag(HTML.Tag tag,
                MutableAttributeSet aset, int pos) {
            if (tag.equals(HTML.Tag.A)) {
                String href  = (String)aset.getAttribute(HTML.Attribute.HREF);
                String style = (String)aset.getAttribute(HTML.Attribute.STYLE);
                if (href != null && href.indexOf("show_friend.pl?id=") == 0) {
                    int id = Integer.parseInt(href.substring(18));
                    if (!((Integer)source.id).equals(id)) {
                        idList.add(id);
                        System.err.println("" + id);
                    }
                }
                if (style != null && style.indexOf("background: url(") == 0) {
                    int urlEnd = style.indexOf("); text-indent");
                    String imageURL = style.substring(16, urlEnd);
                    imageList.add(imageURL);
                    System.err.println(imageURL);
                }
            }
            if (tag.equals(HTML.Tag.SPAN)) {
                text = new StringBuffer();                
            }
        }
        
        public void handleSimpleTag(HTML.Tag tag,
                MutableAttributeSet aset, int pos) {
        }

        public void handleEndTag(HTML.Tag tag, int pos) {
            if (tag.equals(HTML.Tag.SPAN)  && !idList.isEmpty()) {
                Integer id = idList.remove();
                System.err.print("" + id);
                String image = imageList.remove();
                System.err.println(": " + image);
                MixiNode node = (MixiNode)findNode(id);
                if (node == null) {
                    node = (MixiNode)addNode(id, text.toString());
                    layouter.locateNewNode(node, source);
                }
                node.setLabel(text.toString());
                if (node.faceUrl == null && image.indexOf("noimage") == -1) {
                    try {
                        node.faceUrl = new URL(image);
                        if (allThumnails) startFetchingThumbnail(node);
                    } catch (Exception e) {
                        ;
                    }
                }
                if (node.z < source.doi - 1)
                    liftUp(node, source.doi - 1);
                if (source != node)
                    addLink(source, node, text.toString());
                text = null;
                mymixies++;
            }
        }

        public void handleText(char[] data, int pos) {
            if (text != null)
                text.append(data);
            // System.err.println(data);
        }
    }

    public void startFetchingThumbnail(NvNode node) {
        if (((MixiNode)node).faceUrl == null)
            return; 
        ThumbnailLoader loader = new ThumbnailLoader((MixiNode)node);
        new Thread(loader).start();
    }
    
    class ThumbnailLoader implements Runnable {
        MixiNode node;
        
        ThumbnailLoader(MixiNode node) {
            this.node = node;
        }
        
        public void run() {
            try {
                node.thumbnail = Toolkit.getDefaultToolkit().createImage(node.faceUrl);
                node.renderingMode = NvNode.THUMBNAIL; 
                // node.renderingCache = null;
                node.changed = true;
                drawer.update();
            } catch (Exception e) {
                ;
            }
        }
    }
}
