/*
 *  nvj - NattoView/Java 
 *  (c) 2005-2007 Hidekazu Shiozawa
 *  http://vilab.org
 */

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.vecmath.*;
import javax.media.j3d.*;
import com.sun.j3d.utils.universe.SimpleUniverse;
import com.sun.j3d.utils.image.TextureLoader;


public class NvDrawerJ3D extends NvDrawer {

    private Canvas3D canvas;

    private SimpleUniverse universe;
    private View view;
    private BranchGroup root, nodeBranch, linkBranch;
    private Appearance linkAppearance = new Appearance();
    // private Appearance nodeAppearance = new Appearance();

    private float transX = 0.0f, transY = 0.0f, transZ = 20.0f;
    private float rotX = 0.0f, rotZ = 0.0f;

    public NvDrawerJ3D(NvChart chart, int xgrid, int ygrid) {
        super(chart, xgrid, ygrid);
    }

    public void update() {
        if (nodeBranch == null || linkBranch == null)
            return;
        for (NvNode node: chart.getNodes()) {
            drawNode(node);
            for (NvLink link: node.getLinks()) {
                drawLink(link);
            }
        }
    }

    public void initDrawer() throws Exception {
        GraphicsConfiguration config = SimpleUniverse.getPreferredConfiguration();
        canvas = new Canvas3D(config);
        canvas.setDoubleBufferEnable(true);
        canvas.setBounds(0, 0, 600, 600);
        frame.getContentPane().add(canvas, BorderLayout.CENTER);

        linkAppearance.setColoringAttributes(new ColoringAttributes(
                new Color3f(linkColor), ColoringAttributes.SHADE_FLAT));
        TransparencyAttributes ta = new TransparencyAttributes();
        ta.setTransparencyMode(TransparencyAttributes.BLENDED);
        ta.setTransparency(0.5f);
        linkAppearance.setTransparencyAttributes(ta);
        LineAttributes la = new LineAttributes();
        la.setLineWidth(1.0f);
        la.setLinePattern(LineAttributes.PATTERN_SOLID);
        linkAppearance.setLineAttributes(la);

        universe = new SimpleUniverse(canvas);
        view = universe.getViewer().getView();
        view.setBackClipDistance(200.0f);
        view.setProjectionPolicy(View.PERSPECTIVE_PROJECTION);

        TransformGroup tgroup = universe.getViewingPlatform()
                .getViewPlatformTransform();
        Transform3D trans = new Transform3D();
        trans.set(new Vector3f(transX, transY, transZ));
        tgroup.setTransform(trans);

        BranchGroup scene = createSceneBranch();
        universe.addBranchGraph(scene);

        canvas.setVisible(true);
        canvas.addKeyListener(this);
        canvas.addMouseListener(this);
        canvas.requestFocus();
        canvas.repaint();
    }

    protected BranchGroup createSceneBranch() throws Exception {
        root = new BranchGroup();
        BoundingSphere bounds = new BoundingSphere();
        bounds.setRadius(1000.0);

        Background bg = new Background(new Color3f(backColor));
        bg.setApplicationBounds(bounds);
        root.addChild(bg);

        AmbientLight ambient = new AmbientLight(new Color3f(0.5f, 0.5f, 0.5f));
        ambient.setEnable(true);
        root.addChild(ambient);

        PointLight light = new PointLight(new Color3f(1.0f, 1.0f, 1.0f),
                new Point3f(0.0f, 0.0f, 1.0f), new Point3f(1.0f, 0.0f, 0.0f));
        light.setEnable(true);
        light.setInfluencingBounds(bounds);
        root.addChild(light);

        nodeBranch = new BranchGroup();
        nodeBranch.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
        nodeBranch.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
        nodeBranch.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
        root.addChild(nodeBranch);

        linkBranch = new BranchGroup();
        linkBranch.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
        linkBranch.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
        linkBranch.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
        root.addChild(linkBranch);

        /*
        float floor[] = { -1.0f, -0.5f, 0.0f, 1.0f, -0.5f, 0.0f, 1.0f, 0.5f,
                0.0f, -1.0f, 0.5f, 0.0f, };
        QuadArray geom = new QuadArray(floor.length / 3, LineArray.COORDINATES);
        geom.setCoordinates(0, floor);
        Shape3D shape = new Shape3D(geom);
        Appearance ap = new Appearance();
        ap.setColoringAttributes(new ColoringAttributes(
                new Color3f(floorColor), ColoringAttributes.SHADE_FLAT));
        shape.setAppearance(ap);
        shape.setPickable(false);
        root.addChild(shape);
        */

        return root;
    }

    protected void drawNode(NvNode node) {
        Transform3D move = new Transform3D();
        move.set(new Vector3f(node.x * (xgrid / ygrid), node.y, node.z));

        if (!node.changed && node.renderingCache != null) {
            Shape3D icon = (Shape3D)node.renderingCache;
            TransformGroup tgroup = (TransformGroup)icon.getParent();
            tgroup.setTransform(move);
            return;
        }

        Texture texture;
        if (node.renderingMode == NvNode.THUMBNAIL) {
            BufferedImage bi = new BufferedImage(xgrid * 2, ygrid * 2,
                    BufferedImage.TYPE_4BYTE_ABGR);
            Graphics2D g2 = (Graphics2D)bi.getGraphics();
            if (g2.drawImage(node.thumbnail, 0, 0, 128, 64, imageObserver)) {
                g2.setColor(labelColor);
                g2.drawRoundRect(0, 0, bi.getWidth() - 1, bi.getHeight() - 1, 8, 8);
                texture = new TextureLoader(bi).getTexture();
                node.thumbnail.flush();
                // node.thumbnail = null;
                node.changed = false;
            } else {
                bi = makeLabelImage(node.getLabel(0), xgrid, ygrid, node.lastScan >= 0);
                texture = new TextureLoader(bi).getTexture();
            }
            g2.dispose();
        } else {
            BufferedImage bi = 
                makeLabelImage(node.getLabel(0), xgrid, ygrid, node.lastScan >= 0);
            texture = new TextureLoader(bi).getTexture();
            node.renderingMode = NvNode.LABEL;
            node.changed = false;
        }
        Appearance ap = new Appearance();
        ap.setTexture(texture);

        Point3f[] vertex = new Point3f[4];
        vertex[0] = new Point3f(0.0f, 0.0f, 0.02f);
        vertex[1] = new Point3f((float)(xgrid / ygrid), 0.0f, 0.02f);
        vertex[2] = new Point3f((float)(xgrid / ygrid), 1.0f, 0.02f);
        vertex[3] = new Point3f(0.0f, 1.0f, 0.02f);
        QuadArray geometry = new QuadArray(vertex.length,
                GeometryArray.COORDINATES | GeometryArray.TEXTURE_COORDINATE_2);
        geometry.setCoordinates(0, vertex);

        TexCoord2f[] texCoord = new TexCoord2f[4];
        texCoord[0] = new TexCoord2f(0.0f, 0.0f);
        texCoord[1] = new TexCoord2f(1.0f, 0.0f);
        texCoord[2] = new TexCoord2f(1.0f, 1.0f);
        texCoord[3] = new TexCoord2f(0.0f, 1.0f);
        geometry.setTextureCoordinates(0, 0, texCoord);

        if (node.renderingCache != null) {
            Shape3D icon = (Shape3D)node.renderingCache;
            icon.setGeometry(geometry);
            icon.setAppearance(ap);
        } else {
            Shape3D icon = new Shape3D(geometry, ap);
            icon.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
            icon.setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);
            icon.setAppearanceOverrideEnable(true);
            icon.setUserData(node);
            icon.setPickable(true);
            node.renderingCache = icon;
            
            TransformGroup tgroup = new TransformGroup(move);
            tgroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
            tgroup.addChild(icon);

            BranchGroup bgroup = new BranchGroup();
            bgroup.setCapability(BranchGroup.ALLOW_DETACH);
            bgroup.addChild(tgroup);
            nodeBranch.addChild(bgroup);
        }
    }

    protected void drawLink(NvLink link) {
        NvNode s = link.getSource();
        NvNode d = link.getDest();
        Point3f points[] = new Point3f[2];
        points[0] =
            new Point3f((s.x + 0.5f) * (xgrid / ygrid), (s.y + 0.5f), s.z + 0.01f);
        points[1] = 
            new Point3f((d.x + 0.5f) * (xgrid / ygrid), (d.y + 0.5f), d.z + 0.01f);

        LineArray geom = new LineArray(points.length, LineArray.COORDINATES);
        geom.setCoordinates(0, points);

        if (link.renderingCache != null) {
            Shape3D shape = (Shape3D)link.renderingCache;
            shape.setGeometry(geom);
            shape.setAppearance(linkAppearance);
        } else  {
            Shape3D shape = new Shape3D(geom, linkAppearance);
            shape.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
            shape.setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);
            shape.setPickable(false);
            shape.setAppearanceOverrideEnable(true);
            link.renderingCache = shape;

            BranchGroup bgroup = new BranchGroup();
            bgroup.setCapability(BranchGroup.ALLOW_DETACH);
            bgroup.addChild(shape);
            linkBranch.addChild(bgroup);
        }
    }

    public void keyTyped(KeyEvent e) {
        switch (e.getKeyChar()) {
        case 27:
            System.exit(0);
/*
        case 'w':
            rotX += 0.005f;
            break;
        case 'z':
            rotX -= 0.005f;
            break;
        case 'a':
            rotZ -= 0.005f;
            break;
        case 's':
            rotZ += 0.005f;
            break;
        case 'j':
            transZ -= 0.1f;
            break;
        case 'k':
            transZ += 0.1f;
            break;
        case 'i':
            transY -= 0.1f;
            break;
        case 'm':
            transY += 0.1f;
            break;
        case 'l':
            chart.liftUp(chart.getFocus());
            break;
        case 'L':
            chart.liftDown(chart.getFocus());
            break;
*/
        case 'x':
            chart.liftDownAll();
            break;
        }
        changed();
        e.consume();
    }

    public void sliderChangedXT(int v) {
        transX = - v;
        changed();
    }
    public void sliderChangedXR(int v) {
        rotZ = - v / 180.0f * (float)Math.PI;        
        changed();
    }
    public void sliderChangedYT(int v) {
        transY = v;
        changed();
    }
    public void sliderChangedYR(int v) {
        rotX = - v / 180.0f * (float)Math.PI;
        changed();
    }
    public void sliderChangedZ(int v) {
        transZ = -v;        
        changed();
    }

    public void sliderChangedLift(int v) {
        v = 5 - v;
        if (v < 0) return;
        NvNode node = chart.getFocus();
        if (node.doi < v)
            chart.liftUp(node, v);
        else if (node.doi > v)
            chart.liftDown(node, v);
        changed();
    }

    public void changed() {
        TransformGroup tgroup =
            universe.getViewingPlatform().getViewPlatformTransform();
        Transform3D trans = new Transform3D();
        Transform3D t = new Transform3D();
        t.rotZ(rotZ);
        trans.mul(t);
        t.rotX(rotX);
        trans.mul(t);
        t.set(new Vector3f(transX, transY, transZ));
        trans.mul(t);
        tgroup.setTransform(trans);
    }
    
    public void mousePressed(MouseEvent e) {
        int x = e.getX();
        int y = e.getY();
        // int b = e.getButton();

        Point3d eye_pos = new Point3d();
        Point3d mouse_pos = new Point3d();
        canvas.getCenterEyeInImagePlate(eye_pos);
        canvas.getPixelLocationInImagePlate(x, y, mouse_pos);

        Transform3D motion = new Transform3D();
        canvas.getImagePlateToVworld(motion);
        motion.transform(eye_pos);
        motion.transform(mouse_pos);

        Vector3d direction = new Vector3d(mouse_pos);
        direction.sub(eye_pos);
        direction.normalize();

        PickRay pickRay = new PickRay(eye_pos, direction);
        SceneGraphPath path = root.pickClosest(pickRay);

        Node icon = path.getObject();
        if (icon instanceof Shape3D) {
            NvNode node = (NvNode)icon.getUserData();
            chart.setFocus(node);
            if (node.lastScan < 0)
                chart.startScanningLinks(node);
        }
    }
}
