/*
 *  nvj - NattoView/Java 
 *  (c) 2005-2007 Hidekazu Shiozawa
 *  http://vilab.org
 */

import java.awt.*;
import java.awt.image.BufferedImage;
import java.nio.*;
import com.sun.opengl.util.texture.*;
import com.sun.opengl.util.*;
import javax.media.opengl.*;
import javax.media.opengl.glu.*;
import java.awt.event.*;

public class NvDrawerJogl extends NvDrawer implements GLEventListener {

    private GLCanvas canvas;
    private GLContext glcontext;
    private GLU glu = new GLU();

    private float transX = 0.0f, transY = 0.0f, transZ = -20.0f;
    private float rotX = 0.0f, rotZ = 0.0f;
    
    private boolean picking = false;
    private IntBuffer selectBuffer;
    private int name = 0;

    public NvDrawerJogl(NvChart chart, int xgrid, int ygrid) {
        super(chart, xgrid, ygrid);
    }

    public void update() {
        if (canvas != null) canvas.repaint();
    }

    public void initDrawer() throws Exception {
        canvas = new GLCanvas();
        canvas.setBounds(0, 0, 600, 600);
        frame.getContentPane().add(canvas, BorderLayout.CENTER);
        frame.getContentPane().doLayout();
        canvas.setVisible(true);
        canvas.addKeyListener(this);
        canvas.addMouseListener(this);
        canvas.addGLEventListener(this);
        canvas.requestFocus();
    }

    public void init(GLAutoDrawable drawable) {
        glcontext = drawable.getContext();
        GL gl = drawable.getGL();
        gl.glShadeModel(GL.GL_SMOOTH);
        gl.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA);
        gl.glDisable(GL.GL_BLEND);

        float color[] = new float[3];
        backColor.getColorComponents(color);
        gl.glClearColor(color[0], color[1], color[2], 0.0f);

        gl.glClearDepth(1.0f);
        gl.glEnable(GL.GL_DEPTH_TEST);
        gl.glDepthFunc(GL.GL_LEQUAL);
        gl.glHint(GL.GL_PERSPECTIVE_CORRECTION_HINT, GL.GL_NICEST);

        ByteBuffer byteBuffer = 
            ByteBuffer.allocateDirect(BufferUtil.SIZEOF_INT * 100);
        byteBuffer.order(ByteOrder.nativeOrder());
        selectBuffer = byteBuffer.asIntBuffer();
    }

    public void display(GLAutoDrawable drawable) {
        GL gl = drawable.getGL();
        drawView(gl);
    }

    public void displayChanged(GLAutoDrawable drawable, boolean a, boolean b) {
    }

    public void reshape(GLAutoDrawable drawable, int x, int y, int w, int h) {
        GL gl = drawable.getGL();
        if (h <= 0) h = 1;
        gl.glMatrixMode(GL.GL_PROJECTION);
        gl.glLoadIdentity();
        setPerspective(w, h);
        gl.glMatrixMode(GL.GL_MODELVIEW);
        canvas.repaint();
    }

    public void setPerspective(int w, int h) {
        double aspect = (double)w / h;
        glu.gluPerspective(50.0, aspect, 0.1, 200.0);
    }
    
    public void drawView(GL gl) {
        gl.glClear(GL.GL_COLOR_BUFFER_BIT);
        gl.glClear(GL.GL_DEPTH_BUFFER_BIT);
        gl.glLoadIdentity();

        gl.glTranslatef(transX, transY, transZ);
        gl.glRotatef(rotX, 1.0f, 0.0f, 0.0f);
        gl.glRotatef(rotZ, 0.0f, 0.0f, 1.0f);

        float color[] = new float[4];
        gridColor.getColorComponents(color);
        gl.glColor4fv(color, 0);
        gl.glLineWidth(1.0f);
        for (int x = -50; x <= 50; x++) {
            gl.glBegin(GL.GL_LINES);
            gl.glVertex3f(x*2, -50, 0f);
            gl.glVertex3f(x*2,  50, 0f);
            gl.glVertex3f(-100, x, 0f);
            gl.glVertex3f( 100, x, 0f);
            gl.glEnd();
        }

        for (NvNode node: chart.getNodes()) {
            drawNode(gl, node);
            for (NvLink link: node.getLinks()) {
                drawLink(gl, link);
            }
        }
    }

    protected void drawNode(GL gl, NvNode node) {
        gl.glPushMatrix();
        gl.glTranslatef(node.x * (xgrid / ygrid), node.y, 0);

        if (!picking) {
            float color[] = new float[4];
            shadowColor.getColorComponents(color);
            gl.glColor4fv(color, 0);
            gl.glBegin(GL.GL_QUADS);
            gl.glVertex3f(0.0f, 0.0f, 0.02f);
            gl.glVertex3f((float)(xgrid / ygrid), 0.0f, 0.02f);
            gl.glVertex3f((float)(xgrid / ygrid), 1.0f, 0.02f);
            gl.glVertex3f(0.0f, 1.0f, 0.02f);
            gl.glEnd();
        }
        
        gl.glTranslatef(0, 0, node.z);

        if (picking) {
            node.pickingId = new Integer(++name);
            gl.glLoadName(name);
            gl.glRotatef(rotX, -1.0f, 0.0f, 0.0f);  /*****/
            gl.glBegin(GL.GL_QUADS);
            gl.glVertex3f(0.0f, 0.0f, 0.02f);
            gl.glVertex3f((float)(xgrid / ygrid), 0.0f, 0.02f);
            gl.glVertex3f((float)(xgrid / ygrid), 1.0f, 0.02f);
            gl.glVertex3f(0.0f, 1.0f, 0.02f);
            gl.glEnd();
            gl.glPopMatrix();
            return;
        }

        float color[] = new float[3];
        nodeColor.getColorComponents(color);
        gl.glColor4fv(color, 0);
        
        Texture texture;
        if (!node.changed && node.renderingCache != null) {
            texture = (Texture)node.renderingCache;
        } else if (node.renderingMode == NvNode.THUMBNAIL) {
            BufferedImage bi = new BufferedImage(xgrid * 2, ygrid * 2,
                    BufferedImage.TYPE_4BYTE_ABGR);
            Graphics2D g2 = (Graphics2D)bi.getGraphics();
            if (g2.drawImage(node.thumbnail, 0, 0, bi.getWidth(), bi.getHeight(),
                    imageObserver)) {
                g2.setColor(labelColor);
                // g2.drawString(node.getLabel(0), 4, 16);
                g2.drawRoundRect(0, 0, bi.getWidth() - 1, bi.getHeight() - 1, 8, 8);
                node.renderingCache = texture = TextureIO.newTexture(bi, false);
                node.thumbnail.flush();
                // node.thumbnail = null;
                node.changed = false;
            } else {
                texture = (Texture)node.renderingCache;
            }
            g2.dispose();
        } else {
            BufferedImage bi = 
                makeLabelImage(node.getLabel(0), xgrid, ygrid, node.lastScan >= 0);
            node.renderingCache = texture = TextureIO.newTexture(bi, false);
            node.renderingMode = NvNode.LABEL;
            node.changed = false;
        }

        texture.enable();
        texture.bind();
        TextureCoords tc = texture.getImageTexCoords();

        // gl.glRotatef(rotZ, 0.0f, 0.0f, -1.0f);  /*****/
        gl.glRotatef(rotX, -1.0f, 0.0f, 0.0f);  /*****/
        gl.glBegin(GL.GL_QUADS);
        gl.glTexCoord2f(tc.left(), tc.bottom());
        gl.glVertex3f(0.0f, 0.0f, 0.02f);
        gl.glTexCoord2f(tc.right(), tc.bottom());
        gl.glVertex3f((float)(xgrid / ygrid), 0.0f, 0.02f);
        gl.glTexCoord2f(tc.right(), tc.top());
        gl.glVertex3f((float)(xgrid / ygrid), 1.0f, 0.02f);
        gl.glTexCoord2f(tc.left(), tc.top());
        gl.glVertex3f(0.0f, 1.0f, 0.02f);
        gl.glEnd();
        
        texture.disable();
        gl.glPopMatrix();
    }

    protected void drawLink(GL gl, NvLink link) {
        NvNode s = link.getSource();
        NvNode d = link.getDest();

        float color[] = new float[4];
        if (chart.focus == s) {
            linkColor2.getComponents(color);
            gl.glLineWidth(1.0f);
        } else {
            linkColor.getComponents(color);
            gl.glLineWidth(1.0f);
        }
        gl.glColor4fv(color, 0);
        
        gl.glEnable(GL.GL_BLEND);
        //gl.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_DST_ALPHA);
        //gl.glDepthMask(false);

        gl.glBegin(GL.GL_LINES);
        gl.glVertex3f((s.x + 0.5f) * (xgrid / ygrid), (s.y + 0.5f), s.z + 0.01f);
        gl.glVertex3f((d.x + 0.5f) * (xgrid / ygrid), (d.y + 0.5f), d.z + 0.01f);
        gl.glEnd();

        //gl.glDepthMask(true);   
        gl.glDisable(GL.GL_BLEND);        

        /*
        if (s.z != 0 || d.z != 0) {
            shadowColor.getComponents(color);
            gl.glColor4fv(color, 0);
            gl.glLineWidth(1.0f);
        
            gl.glBegin(GL.GL_LINES);
            gl.glVertex3f((s.x + 0.5f) * (xgrid / ygrid), (s.y + 0.5f), 0f);
            gl.glVertex3f((d.x + 0.5f) * (xgrid / ygrid), (d.y + 0.5f), 0f);
            gl.glEnd();
        }
        */
    }

    public int pick(int xmouse, int ymouse) {
        glcontext.makeCurrent();
        GL gl = glcontext.getGL();

        double x = xmouse, y = ymouse;
        gl.glSelectBuffer(selectBuffer.capacity(), selectBuffer);
        gl.glRenderMode(GL.GL_SELECT);
        gl.glInitNames();

        gl.glMatrixMode(GL.GL_PROJECTION);
        gl.glPushMatrix();

        gl.glLoadIdentity();
        IntBuffer viewport = IntBuffer.allocate(4);
        gl.glGetIntegerv(GL.GL_VIEWPORT, viewport);
        glu.gluPickMatrix(x, viewport.get(3) - y, 5.0, 5.0, viewport);
        setPerspective(viewport.get(2), viewport.get(3));

        gl.glMatrixMode(GL.GL_MODELVIEW);
        gl.glLoadIdentity();
        gl.glTranslatef(transX, transY, transZ);
        gl.glRotatef(rotX, 1.0f, 0.0f, 0.0f);
        gl.glRotatef(rotZ, 0.0f, 0.0f, 1.0f);

        picking = true;
        name = 0;
        gl.glPushName(name);
        //synchronized (chart.nodes) {
        for (NvNode node: chart.getNodes()) {
            drawNode(gl, node);
        }
        //}
        picking = false;

        gl.glMatrixMode(GL.GL_PROJECTION);
        gl.glPopMatrix();
        gl.glMatrixMode(GL.GL_MODELVIEW);
        gl.glFlush();
        int hits = gl.glRenderMode(GL.GL_RENDER);
        return processHits(hits, selectBuffer);
    }

    protected int processHits(int hits, IntBuffer buffer) {
        int offset = 0;
        float zmin = 10000f;
        int zname = 0;
        for (int i = 0; i < hits; i++) {
            int names = buffer.get(offset++);
            buffer.get(offset++); // z1
            float z2 = (float)buffer.get(offset++) / 0x7fffffff;
            if (z2 < zmin)
                zmin = z2;
            /*
            System.err.println(" hit: " + (i + 1));
            System.err.println(" number of names: " + names);
            System.err.println(" z1: " + z1);
            System.err.println(" z2: " + z2);
            System.err.println(" names: ");
            */
            for (int j = 0; j < names; j++) {
                int name = buffer.get(offset++);
                if (j == (names - 1)) {
                    if (z2 == zmin)
                        zname = name;
                    // System.err.print("*");
                }
                // System.err.println(name);
            }
        }
        return zname;
    }

    public void keyTyped(KeyEvent e) {
        switch (e.getKeyChar()) {
        case 27:
            System.exit(0);
/*
        case 'z':
            rotX += 1.0f;
            break;
        case 'w':
            rotX -= 1.0f;
            break;
        case 's':
            rotZ -= 1.0f;
            break;
        case 'a':
            rotZ += 1.0f;
            break;
        case 'k':
            transZ -= 1.0f;
            break;
        case 'j':
            transZ += 1.0f;
            break;
        case 'm':
            transY -= 1.0f;
            break;
        case 'i':
            transY += 1.0f;
            break;
        case 'l':
            chart.liftUp(chart.getFocus());
            break;
        case 'L':
            chart.liftDown(chart.getFocus());
            break;
*/
        case 'x':
            chart.liftDownAll();
            break;
        }
        e.consume();
        canvas.repaint();
    }
    
    public void sliderChangedXT(int v) {
        transX = - v;
    }
    public void sliderChangedXR(int v) {
        rotZ = v;        
    }
    public void sliderChangedYT(int v) {
        transY = v;
    }
    public void sliderChangedYR(int v) {
        rotX = v;
    }
    public void sliderChangedZ(int v) {
        transZ = v;        
    }

    public void sliderChangedLift(int v) {
        v = 5 - v;
        if (v < 0) return;
        NvNode node = chart.getFocus();
        if (node.doi < v)
            chart.liftUp(node, v);
        else if (node.doi > v)
            chart.liftDown(node, v);
    }
    
    public void mousePressed(MouseEvent e) {
        int nn = pick(e.getX(), e.getY());
        // System.err.println(nn);

        NvNode found = null;
        for (NvNode node: chart.getNodes()) {
            if (node.pickingId == null) continue;
            int n = ((Integer)node.pickingId).intValue();
            // System.err.println(n);
            if (n == nn) {
                found = node;
                break;
            }
        }
        if (found != null) {
            chart.setFocus(found);
            if (found.lastScan < 0)
                chart.startScanningLinks(found);
        }
        e.consume();
    }
}
