/*
 *  nvj - NattoView/Java 
 *  (c) 2005-2007 Hidekazu Shiozawa
 *  http://vilab.org
 */

import java.util.*;

public class NvLayoutHilbertCurve extends NvLayout {

    private final int RUL = 0, DLU = 1, LDR = 2, URD = 3;
    private int x, y, L;
    private float spacing;
    private int count;
    private NvNode nextNode;
    private Iterator iter;

    private SortedSet<NvNode> set = null;
    
    public int layout(NvChart chart) {
        int n = chart.nodeNumbers() /* + n * 6 */;
        if (n == 0) return 1; 
        int level = (int)Math.ceil(Math.log(n) / Math.log(4));
        level = Math.max(level, 4);
        dimension = 1 << level;
        spacing = (float)(dimension * dimension + 1) / n;

        x = -dimension / 2;
        y = dimension / 2;
        L = 0;
        if (set == null) {
            set = new TreeSet<NvNode>(chart.comparator);
        }
        if (set.size() != n) {
            set.clear();
            for (NvNode node: chart.getNodes()) {
                set.add(node);
            }
        }
        iter = set.iterator();
        nextNode = (NvNode)iter.next();
        count = 0;
        hilcurve(RUL, level);
        return dimension;
    }

    private void hilcurve(int dir, int level) {
        if (nextNode == null)
            return;
        if (level == 0) {
            L++;
            if (L >= count * spacing) {
                count++;
                nextNode.xTarget = x;
                nextNode.yTarget = y;
                nextNode.zTarget = nextNode.z = nextNode.doi * dimension / 5.0f;
                nextNode = iter.hasNext() ? (NvNode)iter.next() : null;
            }
            return;
        }
        switch (dir) {
        case RUL:
            hilcurve(URD, level - 1);  x++; // 
            hilcurve(RUL, level - 1);  y--; // 
            hilcurve(RUL, level - 1);  x--; // 
            hilcurve(DLU, level - 1);  break;
        case DLU:
            hilcurve(LDR, level - 1);  y++; // 
            hilcurve(DLU, level - 1);  x--; // 
            hilcurve(DLU, level - 1);  y--; // 
            hilcurve(RUL, level - 1);  break;
        case LDR:
            hilcurve(DLU, level - 1);  x--; // 
            hilcurve(LDR, level - 1);  y++; // 
            hilcurve(LDR, level - 1);  x++; // 
            hilcurve(URD, level - 1);  break;
        case URD:
            hilcurve(RUL, level - 1);  y--; // 
            hilcurve(URD, level - 1);  x++; // 
            hilcurve(URD, level - 1);  y++; // 
            hilcurve(LDR, level - 1);  break;
        }
    }
}
