/*
 *  nvj - NattoView/Java 
 *  (c) 2005-2007 Hidekazu Shiozawa
 *  http://vilab.org
 */

public class NvLayoutSpring extends NvLayout {

    int iter = 0;

    public int layout(NvChart chart) {
        dimension = Math.max(16, (int) (Math.sqrt(chart.nodeNumbers()) * 1.5));
        double L1 = 1.0;
        double L2;
        double k1 = 0.1;
        double k2 = 0.02;
        double g = 0.1;

        // synchronized (chart.nodes) {
        float gx = 0, gy = 0;
        for (NvNode v1 : chart.getNodes()) {
            gx += v1.x;
            gy += v1.y;
            v1.fx = v1.fy = 0.0f;
        }
        for (NvNode v1 : chart.getNodes()) {
            for (NvLink link : v1.getRevLinks()) {
                NvNode v2 = link.getSource();
                if (v1 == v2)
                    continue;

                L2 = Math.ceil(Math.sqrt(v1.linkNumbers()))
                        + Math.ceil(Math.sqrt(v2.linkNumbers()));
                double dx = v1.x - v2.x;
                double dy = v1.y - v2.y;
                double d2 = dx * dx + dy * dy;
                if (d2 < 0.01) {
                    locateNewNode(v2, v1);
                    continue;
                }
                double d = Math.sqrt(d2);
                if (d < L1) {
                    v1.fx += k1 * (L1 - d) * (dx / d);
                    v1.fy += k1 * (L1 - d) * (dy / d);
                } else {
                    v1.fx += k2 * (L1 - d) * (dx / d);
                    v1.fy += k2 * (L1 - d) * (dy / d);
                    if (d > L2) {
                        v1.fx += k1 * (L2 - d) * (dx / d);
                        v1.fy += k1 * (L2 - d) * (dy / d);
                    }
                }
            }
            for (NvNode v2 : chart.getNodes()) {
                if (v1 == v2)
                    continue;

                double dx = v1.x - v2.x;
                double dy = v1.y - v2.y;
                double d2 = dx * dx + dy * dy;
                if (d2 < 0.01) {
                    locateNewNode(v2, v1);
                    continue;
                }
                double d = Math.sqrt(d2);
                v1.fx += g / d2 * (dx / d);
                v1.fy += g / d2 * (dy / d);
            }
        }
        for (NvNode v1 : chart.getNodes()) {
            v1.xTarget = v1.x + v1.fx - gx / chart.nodeNumbers();
            v1.yTarget = v1.y + v1.fy - gy / chart.nodeNumbers();
            v1.zTarget = v1.z = v1.doi * dimension / 5.0f;
            // System.err.println(v1);
            // System.err.println("" + v1.xTarget + "," + v1.yTarget);
        }
        // }
        return dimension;
    }

    public boolean animation(NvChart chart) {
        if (iter % 10 == 0)
            layout(chart);
        return super.animation(chart);
    }
}
