/*
 *  nvj - NattoView/Java 
 *  (c) 2005-2007 Hidekazu Shiozawa
 *  http://vilab.org
 */

import java.io.*;
import java.net.*;
import java.util.*;
import javax.swing.text.*;
import javax.swing.text.html.*;
import javax.swing.text.html.parser.*;

class StringComparator implements Comparator<Object> {
    public int compare(Object s1, Object s2) {
        return ((String)s1).compareTo((String)s2);
    }
}

public class WikipediaChart extends NvChart {

    public WikipediaChart(NvLayout layout) {
        super(layout);
        this.comparator = new Comparator<NvNode>() {
            public int compare(NvNode v1, NvNode v2) {
                return ((String)v1.getId()).compareTo((String)v2.getId());
            }
        };
    }

    protected NvNode createNode(Object id, String label) {
        WikipediaNode node = new WikipediaNode(this, (String)id, label);
        return node;
    }

    protected NvLink createLink(NvNode source, NvNode dest, String label) {
        return new WikipediaLink((WikipediaNode)source, (WikipediaNode)dest, label);
    }

    public void startScanningLinks(NvNode node) {
        LinkScanner scanner = new LinkScanner((WikipediaNode)node);
        new Thread(scanner).start();
    }

    class LinkScanner implements Runnable {
        WikipediaNode source;
        
        LinkScanner(WikipediaNode node) {
            this.source = node;
        }
        
        public void run() {
            source.lastScan = System.currentTimeMillis();
            try {
                final URL url = new URL("http://ja.wikipedia.org/wiki/"
                        + URLEncoder.encode(source.getId().toString(), "UTF8"));
                HttpURLConnection conn = (HttpURLConnection)url.openConnection();
                conn.setRequestMethod("GET");
                conn.setRequestProperty("User-Agent", NvSettings.userAgent);
                // conn.setDoOutput(true);

                /*
                System.err.println(conn.getHeaderField(0));
                String key, field;
                for (int i = 1; (key = conn.getHeaderFieldKey(i)) != null; i++) {
                    field = conn.getHeaderField(i);
                    System.err.println(key + ": " + field);
                }
                */
                BufferedReader in = new BufferedReader(new InputStreamReader(conn
                        .getInputStream(), "UTF8"));
                System.err.println("" + url + " : analyzing...");

                WikipediaParserCallback parserCallback =
                    new WikipediaParserCallback(source, url);
                new ParserDelegator().parse(in, parserCallback, true);

                in.close();
                conn.disconnect();
                System.err.println("" + url + " : done!");

            } catch (MalformedURLException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }
            layout();
        }
    }

    class WikipediaParserCallback extends HTMLEditorKit.ParserCallback {
        WikipediaNode source;
        URL base;
        StringBuffer text = null;

        WikipediaParserCallback(WikipediaNode node, URL base) {
            this.source = node;
            this.base = base;
        }
        
        public void handleStartTag(HTML.Tag tag,
                MutableAttributeSet aset, int pos) {
            if (tag.equals(HTML.Tag.A)) {
                String href = (String)aset.getAttribute(HTML.Attribute.HREF);
                String title = (String)aset.getAttribute(HTML.Attribute.TITLE);
                if (href == null || href.indexOf("/wiki/") != 0
                        || title == null || title.equals("")) {
                    href = null;
                    return;
                }
                for (int i = 0; i < NvSettings.wikipediaBlocks.length; i++) {
                    if (NvSettings.wikipediaBlocks[i].matcher(title).matches()) {
                        href = null;
                        return;
                    }
                }
                WikipediaNode node = (WikipediaNode)findNode(title);
                if (node == null) {
                    node = (WikipediaNode)addNode(title, title);
                    layouter.locateNewNode(node, source);
                }
                if (node.z < source.doi - 1)
                    liftUp(node, source.doi - 1);
                if (source != node)
                    addLink(source, node, title);
                    href = null;
            } else if (tag.equals(HTML.Tag.H1)) {
                text = new StringBuffer();
            }
        }

        public void handleEndTag(HTML.Tag tag, int pos) {
            if (tag.equals(HTML.Tag.H1) && text != null) {
                source.setLabel(text.toString());
                text = null;
            }
        }

        public void handleSimpleTag(HTML.Tag tag,
                MutableAttributeSet aset, int pos) {
            ;
        }

        public void handleText(char[] data, int pos) {
            // content.append(data);
            if (text != null)
                text.append(data);
        }

        public void flush() {
            // source.content = content.toString();
            // content = null;
        }
    }
    
    public void startFetchingThumbnail(NvNode node) {
        return;
    }
}

