/*
 *  nvj - NattoView/Java 
 *  (c) 2005-2007 Hidekazu Shiozawa
 *  http://vilab.org
 */

import java.net.URL;
import javax.swing.JFrame;


public class nvj {

    static final int WWW = 0, MIXI = 1, WIKIPEDIA = 2; 
    static final int SPRING_FR = 0, SPRING = 1, HILBERT = 2;
    static final int JOGL = 0, J2D = 1, J3D = 2;
    
    static int mode = WWW;
    static String start = null;
    static int layoutMode = SPRING_FR;
    static int drawerMode = JOGL;
    static int xgrid = 64, ygrid = 32;

    public static void main(String[] args) throws Exception {
        
        NvSettings.loadProperties();
        
        for (int i = 0; i < args.length; i++) {
            System.err.println(args[i]);
            if (args[i].equals("-mixi")) {
                mode = MIXI;
            } else if (args[i].equals("-wikipedia")) {
                mode = WIKIPEDIA;
            } else if (args[i].equals("-jogl")) {
                drawerMode = JOGL;
            } else if (args[i].equals("-j2d")) {
                drawerMode = J2D;
            } else if (args[i].equals("-j3d")) {
                drawerMode = J3D;
            } else if (args[i].equals("-fr")) {
                layoutMode = SPRING_FR;
            } else if (args[i].equals("-spring")) {
                layoutMode = SPRING;
            } else if (args[i].equals("-hilbert")) {
                layoutMode = HILBERT;
            } else {
                start = args[i];
            }
        }

        NvLayout layout;
        switch (layoutMode) {
        case SPRING_FR:
        default:
            layout = new NvLayoutSpringFR();
            break;
        case SPRING:
            layout = new NvLayoutSpring();
            break;
        case HILBERT:
            layout = new NvLayoutHilbertCurve();
            break;
        }

        NvChart chart;
        switch (mode) {
        case WWW:
        default:
            chart = new WebChart(layout);
            break;
        case MIXI:
            chart = new MixiChart(layout, NvSettings.mixiEmail, NvSettings.mixiPassword);
            break;
        case WIKIPEDIA:
            chart = new WikipediaChart(layout);
            break;
        }

        NvDrawer nv;
        switch (drawerMode) {
        case JOGL:
        default:
            nv = new NvDrawerJogl(chart, xgrid, ygrid);
            break;
        case J3D:
            nv = new NvDrawerJ3D(chart, xgrid, ygrid);
            break;
        case J2D:
            nv = new NvDrawerJ2D(chart, xgrid, ygrid);
            break;
        }
        nv.frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        nv.initDrawer();
        chart.setDrawer(nv);

        NvNode node;
        switch (mode) {
        case WWW:
        default:
            if (start == null) 
                start = NvSettings.webStart;
            node = chart.addNode(new URL(start), null);
            break;
        case MIXI:
            if (start == null)
                start = NvSettings.mixiStart;
            node = chart.addNode(Integer.parseInt(start), null);
            break;
        case WIKIPEDIA:
            if (start == null)
                start = NvSettings.wikipediaStart;
            node = chart.addNode(start, null);
            break;
        }
        nv.chart.startScanningLinks(node);
    }
}
